/* jspnm_Pike_F100_image.h - read AVC PIKE F-100 raw images. */
/* Last edited on 2010-06-10 02:51:58 by stolfi */

#ifndef jspnm_Pike_F100_image_H
#define jspnm_Pike_F100_image_H

#include <jspnm.h>
#include <jspnm_image.h>

pnm_image_t *pnm_Pike_F100_raw16_image_read(char *name, bool_t verbose);
  /* Reads file called "{name}", assumed to contain an AVC PIke F-100
    image in RAW16 mode. Returns it as a {pnm_image_t} with 
    {cols = rows = 1000}, {chns = 1}, {maxval = 2^16-1}. 
    If {verbose} is TRUE, prints a notice to {stderr}.
    
    Also reads (and prints, if {verbose} is true) the 
    2944 byte trailer of the image file.
    
    The single channel 0 contain R, G, and B pixels
    interleaved in the 2x2 pattern ((G,R),(B,G)). */

pnm_image_t *pnm_Pike_F100_raw16_image_fread (FILE *rd);
  /* Same as {pnm_Pike_F100_raw16_image_read} but reads from the open file {rd}.
    Does not read the 2944 byte trailer of the image file. */

void pnm_Pike_F100_raw16_image_trailer_read(FILE *rd, bool_t verbose);
  /* Reads (and prints, if {verbose} is true) the 2944 byte 
    trailer of the image file. */

int pnm_Pike_F100_raw16_sample_channel(int col, int row);
  /* Returns the channel index (0=R, 1=G, 2=B) of the pixel in column
    {col} and row [row}, according to the Pike F-100 Bayer screen. */
  
pnm_image_t *pnm_Pike_F100_raw16_debayer(pnm_image_t *img, bool_t verbose);
  /* Applies plain debayering t a Raw16 Pike F-100 image.
  
    The image {img} must have {img.chns==1}. The procedure assumes the
    samples of {img} are R,G,B values interleved in the Bayer pattern
    above. It outputs a three-channel image file {omg} where each
    sampleof {img} has been copied into the appropriate channel. The
    other samples in {omg} are set to zero. */
  
void pnm_Pike_F100_raw16_color_balance(pnm_image_t *img, float bal[], bool_t verbose);
  /* Multiplies by {bal[k]} the samples in each channel {k} of {img}.
    If {img} has three channels, assumes it has been de-bayerized.
    If {img} has only one channel, assumes the three channels are
    interleaved in the Bayer pattern. */
  
void pnm_Pike_F100_raw16_interpolate_bayer_pixel(pnm_image_t *img, int col, int row);
  /* Requires {img.chns == 3} and assumes that it is the result of
    {pnm_Pike_F100_raw16_debayer}. Computes the missing samples of
    pixel in column {col} and row {row} by interpolation of nearby
    ones. */

void pnm_Pike_F100_raw16_interpolate_bayer(pnm_image_t *img, bool_t verbose);
  /* Applies {pnm_Pike_F100_raw16_interpolate_bayer_pixel} to all pixels of {img}. */

#endif
